@@ -9,30 +9,31 @@ import Control.Applicative
 import Control.Exception.IOChoice.Lifted
 import Control.Monad.IO.Class (liftIO)
 import Data.ByteString (ByteString)
-import qualified Data.ByteString.Char8 as BS (pack, concat)
+import Data.Maybe
+import qualified Data.ByteString.Char8 as BS (concat)
 import qualified Data.ByteString.Lazy.Char8 as BL (length)
-import Data.Conduit
 import Network.HTTP.Types
 import Network.Wai
+import Network.Wai.Internal
 import Network.Wai.Application.Classic.Field
 import Network.Wai.Application.Classic.FileInfo
+import Network.Wai.Application.Classic.Header
 import Network.Wai.Application.Classic.Path
 import Network.Wai.Application.Classic.Status
 import Network.Wai.Application.Classic.Types
-import Prelude hiding (catch)
 
 ----------------------------------------------------------------
 
-type Rsp = ResourceT IO RspSpec
+type Rsp = IO RspSpec
 
 ----------------------------------------------------------------
 
-data HandlerInfo = HandlerInfo FileAppSpec Request Path [Lang]
+data HandlerInfo = HandlerInfo FileAppSpec Request IndexedHeader Path [Lang]
 
-langSuffixes :: Request -> [Lang]
-langSuffixes req = map (\x -> (<.> x)) langs ++ [id, (<.> "en")]
+langSuffixes :: IndexedHeader -> [Lang]
+langSuffixes reqidx = map (\x -> (<.> x)) langs ++ [id, (<.> "en")]
   where
-    langs = map fromByteString $ languages req
+    langs = map fromByteString $ languages reqidx
 
 ----------------------------------------------------------------
 
@@ -56,7 +57,7 @@ If-Modified-Since:, Range:, If-Range:, If-Unmodified-Since:.
 -}
 
 fileApp :: ClassicAppSpec -> FileAppSpec -> FileRoute -> Application
-fileApp cspec spec filei req = do
+fileApp cspec spec filei req respond = do
     RspSpec st body <- case method of
         Right GET  -> processGET  hinfo ishtml rfile
         Right HEAD -> processHEAD hinfo ishtml rfile
@@ -66,45 +67,41 @@ fileApp cspec spec filei req = do
             BodyStatus             -> bodyStatus st
             BodyFileNoBody hdr     -> bodyFileNoBody st hdr
             BodyFile hdr afile rng -> bodyFile st hdr afile rng
-    liftIO $ logger cspec req st mlen
-    return response
+    logger cspec req st mlen
+    respond response
   where
-    hinfo = HandlerInfo spec req file langs
+    reqidx = indexRequestHeader (requestHeaders req)
+    hinfo = HandlerInfo spec req reqidx file langs
     method = parseMethod $ requestMethod req
     path = pathinfoToFilePath req filei
     file = addIndex spec path
     ishtml = isHTML spec file
     rfile = redirectPath spec path
-    langs = langSuffixes req
-    zdater = dater cspec
-    noBody st = do
-        hdr <- liftIO . addDate zdater $ addServer cspec []
-        return (responseLBS st hdr "", Nothing)
+    langs = langSuffixes reqidx
+    noBody st = return (responseLBS st (addServer cspec []) "", Nothing)
     bodyStatus st = liftIO (getStatusInfo cspec spec langs st)
                 >>= statusBody st
     statusBody st StatusNone = noBody st
-    statusBody st (StatusByteString bd) = do
-        hdr <- liftIO . addDate zdater $ addServer cspec textPlainHeader
+    statusBody st (StatusByteString bd) =
         return (responseLBS st hdr bd, Just (len bd))
       where
         len = fromIntegral . BL.length
-    statusBody st (StatusFile afile len) = do
-        hdr <- liftIO . addDate zdater $ addServer cspec textHtmlHeader
+        hdr = addServer cspec textPlainHeader
+    statusBody st (StatusFile afile len) =
         return (ResponseFile st hdr fl mfp, Just len)
       where
-        mfp = Just (FilePart 0 len)
+        mfp = Just (FilePart 0 len len)
         fl = pathString afile
-    bodyFileNoBody st hdr = do
-        hdr' <- liftIO . addDate zdater $ addServer cspec hdr
-        return (responseLBS st hdr' "", Nothing)
-    bodyFile st hdr afile rng = do
-        hdr' <- liftIO . addDate zdater $ addLength len $ addServer cspec hdr
-        return (ResponseFile st hdr' fl mfp, Just len)
+        hdr = addServer cspec textHtmlHeader
+    bodyFileNoBody st hdr =
+        return (responseLBS st (addServer cspec hdr) "", Nothing)
+    bodyFile st hdr afile rng =
+        return (ResponseFile st (addServer cspec hdr) fl mfp, Just len)
       where
         (len, mfp) = case rng of
             -- sendfile of Linux does not support the entire file
-            Entire bytes    -> (bytes, Just (FilePart 0 bytes))
-            Part skip bytes -> (bytes, Just (FilePart skip bytes))
+            Entire bytes          -> (bytes, Just (FilePart 0 bytes bytes))
+            Part skip bytes total -> (bytes, Just (FilePart skip bytes total))
         fl = pathString afile
 
 ----------------------------------------------------------------
@@ -115,27 +112,28 @@ processGET hinfo ishtml rfile = tryGet      hinfo ishtml
                             ||> return notFound
 
 tryGet :: HandlerInfo -> Bool -> Rsp
-tryGet hinfo@(HandlerInfo _ _ _ langs) True =
+tryGet hinfo@(HandlerInfo _ _ _ _ langs) True =
     runAnyOne $ map (tryGetFile hinfo True) langs
 tryGet hinfo False = tryGetFile hinfo False id
 
 tryGetFile :: HandlerInfo -> Bool -> Lang -> Rsp
-tryGetFile (HandlerInfo spec req file _) ishtml lang = do
+tryGetFile (HandlerInfo spec req reqidx file _) ishtml lang = do
     finfo <- liftIO $ getFileInfo spec (lang file)
     let mtime = fileInfoTime finfo
-        size = fileInfoSize finfo
+        size  = fileInfoSize finfo
         sfile = fileInfoName finfo
-        date = fileInfoDate finfo
+        date  = fileInfoDate finfo
+        mrange = requestHeaderRange req
         hdr = newHeader ishtml (pathByteString file) date
-        Just pst = ifmodified req size mtime -- never Nothing
-               <|> ifunmodified req size mtime
-               <|> ifrange req size mtime
-               <|> unconditional req size mtime
+        Just pst = ifmodified    reqidx size mtime mrange
+               <|> ifunmodified  reqidx size mtime mrange
+               <|> ifrange       reqidx size mtime mrange
+               <|> unconditional reqidx size mtime mrange
     case pst of
         Full st
           | st == ok200  -> return $ RspSpec ok200 (BodyFile hdr sfile (Entire size))
           | otherwise    -> return $ RspSpec st (BodyFileNoBody hdr)
-        Partial skip len -> return $ RspSpec partialContent206 (BodyFile hdr sfile (Part skip len))
+        Partial skip len -> return $ RspSpec partialContent206 (BodyFile hdr sfile (Part skip len size))
 
 ----------------------------------------------------------------
 
@@ -145,18 +143,19 @@ processHEAD hinfo ishtml rfile = tryHead     hinfo ishtml
                              ||> return notFoundNoBody
 
 tryHead :: HandlerInfo -> Bool -> Rsp
-tryHead hinfo@(HandlerInfo _ _ _ langs) True =
+tryHead hinfo@(HandlerInfo _ _ _ _ langs) True =
     runAnyOne $ map (tryHeadFile hinfo True) langs
 tryHead hinfo False= tryHeadFile hinfo False id
 
 tryHeadFile :: HandlerInfo -> Bool -> Lang -> Rsp
-tryHeadFile (HandlerInfo spec req file _) ishtml lang = do
+tryHeadFile (HandlerInfo spec req reqidx file _) ishtml lang = do
     finfo <- liftIO $ getFileInfo spec (lang file)
     let mtime = fileInfoTime finfo
-        size = fileInfoSize finfo
-        date = fileInfoDate finfo
+        size  = fileInfoSize finfo
+        date  = fileInfoDate finfo
         hdr = newHeader ishtml (pathByteString file) date
-        Just pst = ifmodified req size mtime -- never Nothing
+        mrange = requestHeaderRange req
+        Just pst = ifmodified reqidx size mtime mrange
                <|> Just (Full ok200)
     case pst of
         Full st -> return $ RspSpec st (BodyFileNoBody hdr)
@@ -166,13 +165,13 @@ tryHeadFile (HandlerInfo spec req file _) ishtml lang = do
 
 tryRedirect  :: HandlerInfo -> Maybe Path -> Rsp
 tryRedirect _ Nothing = goNext
-tryRedirect (HandlerInfo spec req _ langs) (Just file) =
+tryRedirect (HandlerInfo spec req reqidx _ langs) (Just file) =
     runAnyOne $ map (tryRedirectFile hinfo) langs
   where
-    hinfo = HandlerInfo spec req file langs
+    hinfo = HandlerInfo spec req reqidx file langs
 
 tryRedirectFile :: HandlerInfo -> Lang -> Rsp
-tryRedirectFile (HandlerInfo spec req file _) lang = do
+tryRedirectFile (HandlerInfo spec req _ file _) lang = do
     _ <- liftIO $ getFileInfo spec (lang file)
     return $ RspSpec movedPermanently301 (BodyFileNoBody hdr)
   where
@@ -184,12 +183,13 @@ redirectHeader = locationHeader . redirectURL
 redirectURL :: Request -> ByteString
 redirectURL req = BS.concat [
     "http://"
-  , serverName req
-  , ":"
-  , (BS.pack . show . serverPort) req
+  -- Host includes ":<port>" if it is not 80.
+  , host
   , rawPathInfo req
   , "/"
   ]
+  where
+    host = fromMaybe "" $ requestHeaderHost req
 
 ----------------------------------------------------------------
 
