@@ -1,13 +1,14 @@
+{-# LANGUAGE CPP #-}
+
 module Network.Wai.Application.Classic.Types where
 
 import Data.ByteString (ByteString)
 import qualified Data.ByteString.Lazy as BL (ByteString)
-import qualified Network.HTTP.Conduit as H
+import qualified Network.HTTP.Client as H
 import Network.HTTP.Date
 import Network.HTTP.Types
+import Network.Wai
 import Network.Wai.Application.Classic.Path
-import Network.Wai.Logger
-import System.Log.FastLogger
 
 ----------------------------------------------------------------
 
@@ -15,9 +16,8 @@ data ClassicAppSpec = ClassicAppSpec {
     -- | Name specified to Server: in HTTP response.
     softwareName :: ByteString
     -- | A function for logging. The third argument is a body size.
-  , logger :: ApacheLogger
-    -- | A function to get the HTTP body of status.
-  , dater :: IO ZonedDate
+  , logger :: Request -> Status -> Maybe Integer -> IO ()
+    -- | A function to get HTTP's GMT Date.
   , statusFileDir :: Path
   }
 
@@ -117,7 +117,9 @@ data Range =
     -- | Entire file showing its file size
     Entire Integer
     -- | A part of a file taking offset and length
-  | Part Integer Integer
+  | Part Integer -- offset
+         Integer -- length
+         Integer -- total
   deriving (Eq,Show)
 
 ----------------------------------------------------------------
